package com.napa.pulse.dao.impl;

import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.*;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcCall;

import java.util.HashMap;
import java.util.Map;

import static org.mockito.Mockito.*;

/**
 * @author Rahul Chaudhary
 * 6/1/2022
 */
@SpringBootTest
@RunWith(MockitoJUnitRunner.class)
public class SessionDAOImplUnitTest {
    @InjectMocks
    SessionDAOImpl sessionDAOImplMock;

    @Mock
    private JdbcTemplate jdbcTemplate;

    @Mock
    private SimpleJdbcCall simpleJdbcCall;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testRecalculateSessionTotals() {
        sessionDAOImplMock = mock(SessionDAOImpl.class);

        Map<String, Object> inParamMap = new HashMap<>();
        inParamMap.put("SESSIONID_IN", 0);
        inParamMap.put("SITEID_IN", 2);

        sessionDAOImplMock.reCalculateSessionTotals(0, 2);

        verify(sessionDAOImplMock, times(1)).reCalculateSessionTotals(0, 2);
    }

    @Test(expected = Exception.class)
    public void testRecalculateSessionTotalsForException() {
        sessionDAOImplMock = mock(SessionDAOImpl.class);

        Map<String, Object> inParamMap = new HashMap<>();
        inParamMap.put("SESSIONID_IN", 0);
        inParamMap.put("SITEID_IN", null);

        SqlParameterSource in = new MapSqlParameterSource(inParamMap);

        SimpleJdbcCall simpleJdbcCall = new SimpleJdbcCall(jdbcTemplate).withProcedureName("RECALC_SESSION_TOTALS");
        when(simpleJdbcCall.execute(in)).thenThrow(Exception.class);
    }
}
