package com.napa.pulse.service.impl;

import com.napa.pulse.dao.interfaces.HubSpokeNetworkDAO;
import com.napa.pulse.dto.ActionResult;
import com.napa.pulse.dto.HubSpokeNetworkDTO;
import com.napa.pulse.dto.HubSpokeWidgetData;
import com.napa.pulse.entity.security.User;
import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;

import static org.assertj.core.api.Assertions.assertThat;

import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

/**
 * @author Rahul Chaudhary
 * 7/12/2022
 */
@SpringBootTest
@RunWith(MockitoJUnitRunner.class)
public class HubSpokeServiceImpUnitTest {

    @InjectMocks
    HubSpokeNetworkServiceImpl hubSpokeNetworkService;

    @Mock
    private HubSpokeNetworkDAO networkDAO;

    @Mock
    private JdbcTemplate jdbcTemplate;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testGetNetwork() {
        List<HubSpokeNetworkDTO> expected = new ArrayList<>();
        expected.add(buildHubSpokeNetworkDTO());
        when(hubSpokeNetworkService.getNetwork(any())).thenReturn(expected);
        List<HubSpokeNetworkDTO> result = hubSpokeNetworkService.getNetwork(buildHubSpokeWidgetData());
        assertThat(result).isNotNull();
        assertThat(result.size()).isEqualTo(1);
    }

    @Test
    public void testGetNetworkById_givenId_givenNullUser() {
        List<HubSpokeNetworkDTO> expected = new ArrayList<>();
        expected.add(buildHubSpokeNetworkDTO());
        when(hubSpokeNetworkService.getNetworkById(any(), any())).thenReturn(expected);
        List<HubSpokeNetworkDTO> result = hubSpokeNetworkService.getNetworkById(101, null);
        assertThat(result).isNotNull();
        assertThat(result.size()).isEqualTo(1);
    }

    @Test
    public void testDeleteNetwork_whenSuccess(){
        ActionResult expected = new ActionResult();
        expected.setSuccess(true);
        when(hubSpokeNetworkService.deleteNetwork(any(),any())).thenReturn(expected);
        ActionResult result = hubSpokeNetworkService.deleteNetwork(101,null);
        assertThat(result).isNotNull();
        assertThat(result.isSuccess()).isEqualTo(true);
    }

    @Test
    public void testDeleteNetwork_whenFailure(){
        ActionResult expected = new ActionResult();
        expected.setSuccess(false);
        when(hubSpokeNetworkService.deleteNetwork(any(),any())).thenReturn(expected);
        ActionResult result = hubSpokeNetworkService.deleteNetwork(101,null);
        assertThat(result).isNotNull();
        assertThat(result.isSuccess()).isEqualTo(false);
    }

    private HubSpokeWidgetData buildHubSpokeWidgetData() {
        HubSpokeWidgetData obj = new HubSpokeWidgetData();
        obj.setUser(buildUser());
        obj.setSelectedHSFilterLoad("showNonEmptyHSNetwork");
        return obj;
    }

    private HubSpokeNetworkDTO buildHubSpokeNetworkDTO() {
        HubSpokeNetworkDTO obj = new HubSpokeNetworkDTO();
        obj.setNetworkId(101);
        obj.setNetworkName("Test-000");
        obj.setNetworkCreatedBy("Rahul Chaudhary");
        obj.setNetworkCreatedDate(null);
        obj.setCurrentUIC(0);
        obj.setCurrentUIC(0);
        obj.setChangeUIC(0);
        obj.setExpanded(false);
        return obj;
    }

    private User buildUser() {
        User testUser = new User();
        testUser.setUserId(8194);
        testUser.setFirstName("Test");
        testUser.setLastName("user");
        testUser.setLogin("login");
        testUser.setRoleId(1);
        testUser.setAccessItemIds(Arrays.asList(1, 2, 3));
        testUser.setLanguage("EN");
        testUser.setBusinessUnit(1);
        testUser.setDeactivated("N");
        testUser.setSiteTypes(Arrays.asList("dc", "sku", "ab"));

        return testUser;
    }

}
