package com.napa.pulse.service.impl;

import com.napa.pulse.dao.interfaces.AcoDAO;
import com.napa.pulse.dao.interfaces.SessionDAO;
import com.napa.pulse.dto.ActionResult;
import com.napa.pulse.utils.informatica.InformaticaServiceClient;
import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.boot.test.context.SpringBootTest;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;

@SpringBootTest
@RunWith(MockitoJUnitRunner.class)
public class SessionServiceImplUnitTest {

    @Mock
    private SessionDAO sessionDAO;

    @Mock
    private AcoDAO acoDAO;

    @Mock
    private InformaticaServiceClient informaticaServiceClient;

    @InjectMocks
    SessionServiceImpl mockServiceImpl;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testFinalizeSession_givenMinMAxSessionCode() {
        ActionResult expected = new ActionResult();
        expected.setSuccess(true);
        when(sessionDAO.isMinMax(anyInt())).thenReturn(true);
        doNothing().when(informaticaServiceClient).startMinMaxWorkflowFinalize();
        doNothing().when(sessionDAO).finalizeMinMaxSession(anyInt(),anyInt(),any(),anyBoolean(),anyInt(),anyBoolean());
        ActionResult actualResult =  mockServiceImpl.finalizeSession(1, 1, null, true);
        assertThat(actualResult.isSuccess()).isEqualTo(true);
    }

    @Test
    public  void testFinalizeSession_givenACOSession(){
        when(sessionDAO.isMinMax(anyInt())).thenReturn(false);
        when(sessionDAO.isACO(anyInt())).thenReturn(true);
        doNothing().when(sessionDAO).finalizeACOSession(anyInt(),anyInt(),any());
        ActionResult actualResult =mockServiceImpl.finalizeSession(1,1,null,false);
          assertNotNull(actualResult);
    }

    @Test
    public  void testFinalizeSession_givenSession(){
        when(sessionDAO.isMinMax(anyInt())).thenReturn(false);
        when(sessionDAO.isACO(anyInt())).thenReturn(false);
        doNothing().when(informaticaServiceClient).startWorkflowFinalize();
        ActionResult actualResult =mockServiceImpl.finalizeSession(1,1,null,false);
        assertNotNull(actualResult);
    }


}
